/*
	2021 year
	Smart pickup  $continue$
*/

#if !defined MAX_SMART_PICKUP_COUNT
	const MAX_SMART_PICKUP_COUNT = 5_000;
#endif

#if !defined SMART_PICKUP_INVALID_ID
	const SMART_PICKUP_INVALID_ID = -1;
#endif

#if !defined MAX_SMART_PICK_DESCRIPTION_TEXT
	const MAX_SMART_PICK_DESCRIPTION_TEXT = 256;
#endif

#if !defined SMART_PICK_DRAW_DIST_3D_LABEL
	const Float: SMART_PICK_DRAW_DIST_3D_LABEL = 15.0;
#endif

#if !defined FLOAT_NAN
	const Float: FLOAT_NAN = (Float:0xFFFFFFFF);
#endif

#tryinclude <foreach>
#tryinclude <math>

#if defined TRYG3D_REMASTERED_INCLUDE
	#if !defined TRYG3D_ENABLE_DRAW3D
		#error "Add macros (define) #define TRYG3D_ENABLE_DRAW3D before #include <3DTryg>"
	#endif
#else
	#if !defined TRYG3D_ENABLE_DRAW3D
		#define TRYG3D_ENABLE_DRAW3D
	#endif
#endif

#tryinclude <3DTryg>

#if !defined TRYG3D_REMASTERED_INCLUDE
	#warning "For debug area need 3DTryg Remastered. Link: https://github.com/AbyssMorgan/3DTryg"
#endif

#if !defined math@GetPolyPointsRotatedRect 
	#error "Need include math.inc (With math@GetPolyPointsRotatedRect / math@GetSemicirclePolyPoint)"
#endif

/*
	enums
*/

enum e_smart_pick_shape_type
{
	SMART_PICK_SEMICYLINDER_TYPE,
	SMART_PICK_CUBOID_TYPE,
	SMART_PICK_CYLINDER_TYPE
};

enum e_smart_pick_action_type
{
	SMART_PICK_TELE_ACTION_TYPE,
	SMART_PICK_HOUSE_ACTION_TYPE,
	SMART_PICK_BUSINESS_ACTION_TYPE,
	SMART_PICK_INFO_ACTION_TYPE
};

static enum e_smart_pick_info
{
	spi_area_id,
	e_smart_pick_shape_type: spi_shape_type,
	spi_pickup_id,
	Text3D: spi_3d_text_id,
	spi_key_id,
	e_smart_pick_action_type: spi_action_type,
	Float: spi_area_size,
	Float: spi_area_angle,
	spi_db_id
};

/*
	vars
*/

static Iterator:g_smart_pickup_iterator<MAX_SMART_PICKUP_COUNT>;
static g_smart_pickup_info[MAX_SMART_PICKUP_COUNT][e_smart_pick_info] = 
{
	{
		-1,
		e_smart_pick_shape_type: -1,
		-1,
		Text3D: -1,
		-1,
		e_smart_pick_action_type: -1,
		FLOAT_NAN,
		FLOAT_NAN,
		-1
	},
	...
};
static g_player_in_smart_pickup[MAX_PLAYERS] = { -1, -1, -1, ... };


/*
	forwards
*/

forward OnPlayerEnterSmartPickup(const playerid, const smart_pickup_id, const key_id);
forward OnPlayerLeaveSmartPickup(const playerid, const smart_pickup_id);
forward e_smart_pick_shape_type:smart_pick@GetShapeType(const smart_pickup_id);
forward Float: smart_pick@GetShapeSize(const smart_pickup_id);
forward Float: smart_pick@GetShapeAngle(const smart_pickup_id);
forward Text3D:smart_pick@Get3DLabelID(const smart_pickup_id);

stock smart_pick@Create(
	const model_pickup,
	const Float: x_pos,
	const Float: y_pos,
	const Float: z_pos,
	const Float: size,
	const e_smart_pick_action_type: action_type,
	const Float: min_z = -FLOAT_INFINITY,
	const Float: max_z = FLOAT_INFINITY,
	const description_text[] = "",
	const e_smart_pick_shape_type: shape_type = SMART_PICK_CUBOID_TYPE,
	const virtual_world = 0,
	const Float: angle = 245.0,
	const interior_id = 0,
	const key_id = -1,
	const Float: text3d_draw_distance = SMART_PICK_DRAW_DIST_3D_LABEL,
	const bool: test_los = false,
	const bool: is_need_debug_area = false
)
{
	new 
		free_idx = Iter_Free(g_smart_pickup_iterator),
		area_id = INVALID_STREAMER_ID;

	if (free_idx == ITER_NONE)
	{
		return SMART_PICKUP_INVALID_ID;
	}
	if (shape_type == SMART_PICK_SEMICYLINDER_TYPE)
	{
		new Float: points[90];
		static Float: null_points[sizeof(points)] = { -1.0, -1.0, -1.0, ... };

		new count = math@GetSemicirclePolyPoint(
			x_pos - (-0.44 * floatsin(angle, degrees)), 
			y_pos + (-0.44 * floatcos(angle, degrees)), 
			angle,
			size,
			points
		);

		if (is_need_debug_area)
		{
			for (new i; i < sizeof(points); i += 2)
			{
				CreateDynamicObject(
					19991, 
					points[i], 
					points[i + 1], 
					z_pos - 0.7,
					0.0,
					0.0,
					0.0
				);
			}
		}
		area_id = CreateDynamicPolygon(
			points, 
			.minz = min_z, 
			.maxz = max_z,
			.worldid = virtual_world, 
			.interiorid = interior_id,
			.maxpoints = count
		);
		points = null_points;
	}
	else if (shape_type == SMART_PICK_CUBOID_TYPE)
	{
		static Float: points[500][2];
		new Float: rect[4][2];
		new Float: temp_poly[sizeof(rect) * sizeof(rect[])];

		math@GetPolyPointsRotatedRect(x_pos, y_pos, angle, size, size, rect);

		for (new i = 0; i < sizeof(rect) * sizeof(rect[]); ++i)
		{
			temp_poly[i] = rect[i / sizeof(rect[])][i % sizeof(rect[])];
		}

		if (is_need_debug_area)
		{
			new count = Tryg3D::DrawPolygon2D(temp_poly, points, .points_per_line = 6);

			for (new i = 0; i < count; i++)
			{
				//printf("__points[%d][0]: %f | __points[%d][1]: %f", i, __points[i][0], i, __points[i][1]);
				// printf("%f, %f, %f", __points[i][0], __points[i][1], z_pos);
				CreateDynamicObject(
					19991, 
					points[i][0],
					points[i][1], 
					z_pos - 0.7,
					0.0,
					0.0,
					0.0
				);
			}
		}
		area_id = CreateDynamicPolygon(
			temp_poly, 
			.minz = min_z, 
			.maxz = max_z,
			.worldid = virtual_world, 
			.interiorid = interior_id,
			.maxpoints = sizeof(temp_poly)
		);
	}
	else if (shape_type == SMART_PICK_CYLINDER_TYPE)
	{
		area_id = CreateDynamicCylinder(
			x_pos, 
			y_pos, 
			min_z, 
			max_z, 
			size, 
			.worldid = virtual_world, 
			.interiorid = interior_id
		);

		if (is_need_debug_area)
		{
			static Float: points[500][3];
			new count = Tryg3D::DrawCircle3D(
				x_pos, 
				y_pos, 
				z_pos - 0.7, 
				size, 
				points,
				.max_sector = 36
			);

			for (new i = 0; i < count; i++)
			{
				CreateDynamicObject(
					19991, 
					points[i][0], 
					points[i][1], 
					z_pos - 0.7,
					0.0,
					0.0,
					0.0
				);
			}
		}
	}
	new pickup_id = CreateDynamicPickup(
		model_pickup, 
		23, 
		x_pos,
		y_pos, 
		z_pos, 
		.worldid = virtual_world, 
		.interiorid = interior_id
	);

	g_smart_pickup_info[free_idx][spi_area_id] = area_id;
	g_smart_pickup_info[free_idx][spi_pickup_id] = pickup_id;
	g_smart_pickup_info[free_idx][spi_shape_type] = shape_type;
	g_smart_pickup_info[free_idx][spi_action_type] = action_type;
	g_smart_pickup_info[free_idx][spi_key_id] = key_id;
	g_smart_pickup_info[free_idx][spi_area_size] = size;
	g_smart_pickup_info[free_idx][spi_area_angle] = angle;

	if (strlen(description_text) > 0)
	{
		g_smart_pickup_info[free_idx][spi_3d_text_id] = CreateDynamic3DTextLabel(
			description_text, 
			-1, 
			x_pos, 
			y_pos, 
			z_pos + 0.3, 
			text3d_draw_distance,  
			.testlos = test_los, 
			.worldid = virtual_world, 
			.interiorid = interior_id
		);
	}
	Iter_Add(g_smart_pickup_iterator, free_idx);
	return free_idx;
}

/*
	hook SA-MP callbacks
*/

/*
	hook OnPlayerEnterDynamicArea
*/

public OnPlayerEnterDynamicArea(playerid, STREAMER_TAG_AREA:areaid)
{
	foreach (new smart_pick_id : g_smart_pickup_iterator)
	{
		if (g_smart_pickup_info[smart_pick_id][spi_area_id] == areaid)
		{
			new key_id = g_smart_pickup_info[smart_pick_id][spi_key_id];
			#if defined debug_mode
				printf("smart_pickup | OnPlayerEnterDynamicArea | key_id: %d | smart_pick_id: %d",
					key_id,
					smart_pick_id
				);
			#endif
			if (key_id == -1)
			{
				g_player_in_smart_pickup[playerid] = smart_pick_id;
				CallRemoteFunction(
					"OnPlayerEnterSmartPickup", 
					"ddd", 
					playerid, 
					smart_pick_id,
					key_id
				);
			}
			break;
		}
	}
#if defined spi_OnPlayerEnterDynamicArea
	return spi_OnPlayerEnterDynamicArea(playerid, areaid);
#else
	return 1;
#endif
}
#if defined _ALS_OnPlayerEnterDynamicArea
	#undef OnPlayerEnterDynamicArea
#else
	#define _ALS_OnPlayerEnterDynamicArea
#endif
#define OnPlayerEnterDynamicArea spi_OnPlayerEnterDynamicArea
#if defined    spi_OnPlayerEnterDynamicArea
	forward spi_OnPlayerEnterDynamicArea(playerid, STREAMER_TAG_AREA:areaid);
#endif 

/*
	hook OnPlayerEnterDynamicArea
*/


public OnPlayerKeyStateChange(playerid, newkeys, oldkeys)
{
	if (IsPlayerInAnyDynamicArea(playerid))
	{
		foreach (new smart_pick_id : g_smart_pickup_iterator)
		{
			//printf("smart_pickup | OnPlayerKeyStateChange | smart_pickup: %d", smart_pick_id);
			if (IsPlayerInDynamicArea(playerid, smart_pick@GetDynAreaID(smart_pick_id)))
			{
				new key_id = g_smart_pickup_info[smart_pick_id][spi_key_id];
				if (key_id != -1)
				{
					if (newkeys & key_id)
					{
						g_player_in_smart_pickup[playerid] = smart_pick_id;
						CallRemoteFunction(
							"OnPlayerEnterSmartPickup", 
							"ddd", 
							playerid, 
							smart_pick_id,
							key_id
						);
					}
				}
				break;
			}
		}
	}
#if defined spi_OnPlayerKeyStateChange
	return spi_OnPlayerKeyStateChange(playerid, newkeys, oldkeys);
#else
	return 1;
#endif
}
#if defined _ALS_OnPlayerKeyStateChange
	#undef OnPlayerKeyStateChange
#else
	#define _ALS_OnPlayerKeyStateChange
#endif
#define OnPlayerKeyStateChange spi_OnPlayerKeyStateChange
#if defined    spi_OnPlayerKeyStateChange
	forward spi_OnPlayerKeyStateChange(playerid, newkeys, oldkeys);
#endif 

/*
	hook OnPlayerLeaveDynamicArea
*/

public OnPlayerLeaveDynamicArea(playerid, STREAMER_TAG_AREA:areaid)
{
	foreach (new smart_pick_id : g_smart_pickup_iterator)
	{
		if (smart_pick@GetDynAreaID(smart_pick_id) == areaid)
		{
			g_player_in_smart_pickup[playerid] = -1;

			CallRemoteFunction(
				"OnPlayerLeaveSmartPickup", 
				"dd", 
				playerid, 
				smart_pick_id
			);
			break;
		}
	}
#if defined spi_OnPlayerLeaveDynamicArea
	return spi_OnPlayerLeaveDynamicArea(playerid, areaid);
#else
	return 1;
#endif
}
#if defined _ALS_OnPlayerLeaveDynamicArea
	#undef OnPlayerLeaveDynamicArea
#else
	#define _ALS_OnPlayerLeaveDynamicArea
#endif
#define OnPlayerLeaveDynamicArea spi_OnPlayerLeaveDynamicArea
#if defined    spi_OnPlayerLeaveDynamicArea
	forward spi_OnPlayerLeaveDynamicArea(playerid, STREAMER_TAG_AREA:areaid);
#endif 

/*
	hook OnPlayerDisconnect
*/

public OnPlayerDisconnect(playerid, reason)
{
	if (g_player_in_smart_pickup[playerid] != -1)
	{
		g_player_in_smart_pickup[playerid] = -1;
	}
#if defined spi_OnPlayerDisconnect
	return spi_OnPlayerDisconnect(playerid, reason);
#else
	return 1;
#endif
}
#if defined _ALS_OnPlayerDisconnect
	#undef OnPlayerDisconnect
#else
	#define _ALS_OnPlayerDisconnect
#endif
#define OnPlayerDisconnect spi_OnPlayerDisconnect
#if defined    spi_OnPlayerDisconnect
	forward spi_OnPlayerDisconnect(playerid, reason);
#endif 

/*
	Check valid smart pickup id
*/

stock smart_pick@IsValidID(const smart_pickup_id)
{
	return Iter_Contains(g_smart_pickup_iterator, smart_pickup_id);
}

/*
	Get shape type smart pickup
*/

stock e_smart_pick_shape_type:smart_pick@GetShapeType(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return e_smart_pick_shape_type:-1;
	}
	return g_smart_pickup_info[smart_pickup_id][spi_shape_type];
}

/*
	Get interior id smart pickup
*/

stock smart_pick@GetInteriorID(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return -1;
	}
	new area_id = g_smart_pickup_info[smart_pickup_id][spi_area_id];
	return Streamer_GetIntData(STREAMER_TYPE_AREA, area_id, E_STREAMER_INTERIOR_ID);
}

/*
	Get world id smart pickup
*/

stock smart_pick@GetWorldID(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return -1;
	}
	new area_id = g_smart_pickup_info[smart_pickup_id][spi_area_id];
	return Streamer_GetIntData(STREAMER_TYPE_AREA, area_id, E_STREAMER_WORLD_ID);
}

/*
	Get model id smart pickup
*/

stock smart_pick@GetModelID(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return -1;
	}
	new pickup_id = g_smart_pickup_info[smart_pickup_id][spi_pickup_id];
	return Streamer_GetIntData(STREAMER_TYPE_PICKUP, pickup_id, E_STREAMER_MODEL_ID);
}

/*
	Get description 3dtext id 
*/

stock smart_pick@Get3DTextLabelID(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return -1;
	}
	return g_smart_pickup_info[smart_pickup_id][spi_3d_text_id];
}

/*
	Set pickup action type
*/

stock smart_pick@SetActionType(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return 0;
	}
	g_smart_pickup_info[smart_pickup_id][spi_action_type];
	return 1;
}

/*
	Get pickup action type
*/

stock e_smart_pick_action_type:smart_pick@GetActionType(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return e_smart_pick_action_type:-1;
	}
	return g_smart_pickup_info[smart_pickup_id][spi_action_type];
}

/*
	Get dynamic area id 
*/

stock smart_pick@GetDynAreaID(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return 0;
	}	
	return g_smart_pickup_info[smart_pickup_id][spi_area_id];
}

/*
	Get 3d label (Text3D)
*/

stock Text3D:smart_pick@Get3DLabelID(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return Text3D:-1;
	}	
	return g_smart_pickup_info[smart_pickup_id][spi_3d_text_id];
}

/*
	Destroy smart pickup
*/

stock smart_pick@Destroy(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return 0;
	}
	Iter_Remove(g_smart_pickup_iterator, smart_pickup_id);
	DestroyDynamicPickup(g_smart_pickup_info[smart_pickup_id][spi_pickup_id]);
	DestroyDynamicArea(g_smart_pickup_info[smart_pickup_id][spi_area_id]);

	g_smart_pickup_info[smart_pickup_id][spi_area_id] = -1;
	g_smart_pickup_info[smart_pickup_id][spi_pickup_id] = -1;
	g_smart_pickup_info[smart_pickup_id][spi_action_type] = e_smart_pick_action_type:-1;
	g_smart_pickup_info[smart_pickup_id][spi_key_id] = -1;
	return 1;
}

stock smart_pick@GetPosition(const smart_pickup_id, &Float: x_pos, &Float: y_pos, &Float: z_pos)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return 0;
	}
	new pickup_id = g_smart_pickup_info[smart_pickup_id][spi_pickup_id];
	
	Streamer_GetItemPos(STREAMER_TYPE_PICKUP, pickup_id, x_pos, y_pos, z_pos);
	return 1;
}

stock smart_pick@SetDataBaseId(const smart_pickup_id, const db_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return 0;
	}
	g_smart_pickup_info[smart_pickup_id][spi_db_id] = db_id;
	return 1;
}

stock smrt_pick@GetDataBaseId(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return 0;
	}
	return g_smart_pickup_info[smart_pickup_id][spi_db_id];
}

stock Float: smart_pick@GetShapeSize(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return FLOAT_NAN;
	}
	return g_smart_pickup_info[smart_pickup_id][spi_area_size];
}

stock Float: smart_pick@GetShapeAngle(const smart_pickup_id)
{
	if (!smart_pick@IsValidID(smart_pickup_id))
	{
		return FLOAT_NAN;
	}
	return g_smart_pickup_info[smart_pickup_id][spi_area_angle];
}

/*
	Get player smart pickup id
*/

stock smart_pick@GetPlayerSmartPickID(const playerid)
{
	return g_player_in_smart_pickup[playerid]; 
}